@@ -2,18 +2,6 @@
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Script to update the year of last modification
 in the license header of source code files.\n
@@ -22,16 +10,18 @@ Also it appends a header if it is missing in the file.
 
 import re
 import sys
-from argparse import ArgumentParser, FileType, Namespace
-from datetime import datetime
+from dataclasses import dataclass
+from functools import cache
 from pathlib import Path
-from subprocess import CalledProcessError, run
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Optional, Sequence, Union
 
-from pontos.terminal import Terminal
+from pontos.errors import PontosError
+from pontos.git import Git
 from pontos.terminal.null import NullTerminal
 from pontos.terminal.rich import RichTerminal
 
+from ._parser import parse_args
+
 SUPPORTED_FILE_TYPES = [
     ".bash",
     ".c",
@@ -47,12 +37,6 @@ SUPPORTED_FILE_TYPES = [
     ".xml",
     ".xsl",
 ]
-SUPPORTED_LICENCES = [
-    "AGPL-3.0-or-later",
-    "GPL-2.0-only",
-    "GPL-2.0-or-later",
-    "GPL-3.0-or-later",
-]
 OLD_LINES = [
     "# \-\*\- coding: utf\-8 \-\*\-",
     "This program is free software: you can redistribute it and/or modify",
@@ -72,6 +56,7 @@ OLD_LINES = [
     "GNU Affero General Public License for more details.",
     "GNU General Public License for more details.",
     "You should have received a copy of the GNU Affero General Public License",
+    "You should have received a copy of the GNU General Public License",
     "along with this program.  If not, see <http://www.gnu.org/licenses/>.",
     "along with this program; if not, write to the Free Software",
     "Foundation, Inc\., 51 Franklin St, Fifth Floor, Boston, MA 02110\-1301 USA\.",  # noqa: E501
@@ -80,34 +65,30 @@ OLD_LINES = [
 
 def _get_modified_year(f: Path) -> str:
     """In case of the changed arg, update year to last modified year"""
-    try:
-        cmd = ["git", "log", "-1", "--format=%ad", "--date=format:%Y", str(f)]
-        proc = run(
-            cmd,
-            text=True,
-            capture_output=True,
-            check=True,
-            universal_newlines=True,
-        )
-        return proc.stdout.rstrip()
-    except CalledProcessError as e:
-        raise e
+    return Git().log("-1", "--date=format:%Y", str(f), format="%ad")[0]
+
+
+@dataclass
+class CopyrightMatch:
+    creation_year: str
+    modification_year: Optional[str]
+    company: str
 
 
 def _find_copyright(
     line: str,
-    regex: re.Pattern,
-) -> Tuple[bool, Union[Dict[str, Union[str, None]], None]]:
-    """Match the line for the regex"""
-    copyright_match = re.search(regex, line)
+    copyright_regex: re.Pattern,
+) -> tuple[bool, Union[CopyrightMatch, None]]:
+    """Match the line for the copyright_regex"""
+    copyright_match = re.search(copyright_regex, line)
     if copyright_match:
         return (
             True,
-            {
-                "creation_year": copyright_match.group(2),
-                "modification_year": copyright_match.group(3),
-                "company": copyright_match.group(4),
-            },
+            CopyrightMatch(
+                creation_year=copyright_match.group(2),
+                modification_year=copyright_match.group(3),
+                company=copyright_match.group(4),
+            ),
         )
     return False, None
 
@@ -135,14 +116,17 @@ def _add_header(
         raise ValueError
 
 
-def _remove_outdated(
-    content: str, cleanup_regexes: List[re.Pattern]
+def _remove_outdated_lines(
+    content: str, cleanup_regexes: list[re.Pattern]
 ) -> Optional[str]:
     """Remove lines that contain outdated copyright header ..."""
     changed = False
     splitted_lines = content.splitlines()
     i = 0
     for line in splitted_lines[:20]:
+        if i > 3 and re.match(r"^(([#*]|//) ?$)", line):
+            splitted_lines.pop(i)
+            continue
         for regex in cleanup_regexes:
             if regex.match(line):
                 changed = True
@@ -151,31 +135,27 @@ def _remove_outdated(
                 break
         i = i + 1
     if changed:
-        return "\n".join(splitted_lines)
+        new_content = "\n".join(splitted_lines) + "\n"
+        return new_content
     return None
 
 
-def _update_file(
+def update_file(
     file: Path,
-    regex: re.Pattern,
-    parsed_args: Namespace,
-    term: Terminal,
-    cleanup_regexes: Optional[List[re.Pattern]] = None,
-) -> int:
-    """Function to update the given file.
-    Checks if header exists. If not it adds an
-    header to that file, else it checks if year
-    is up to date
+    year: str,
+    license_id: str,
+    company: str,
+    *,
+    cleanup: bool = False,
+) -> None:
+    """Function to update the header of the given file
+
+    Checks if header exists. If not it adds an header to that file, otherwise it
+    checks if year is up to date
     """
 
-    if parsed_args.changed:
-        try:
-            parsed_args.year = _get_modified_year(file)
-        except CalledProcessError:
-            term.warning(
-                f"{file}: Could not get date of last modification"
-                f" using git, using {str(parsed_args.year)} instead."
-            )
+    copyright_regex = _compile_copyright_regex()
+    cleanup_regexes = _compile_outdated_regex() if cleanup else None
 
     try:
         with file.open("r+") as fp:
@@ -186,26 +166,27 @@ def _update_file(
                 if line == "":
                     i = 0
                     continue
-                found, copyright_match = _find_copyright(line=line, regex=regex)
+                found, copyright_match = _find_copyright(
+                    line=line, copyright_regex=copyright_regex
+                )
                 i = i - 1
             # header not found, add header
             if i == 0 and not found:
                 try:
                     header = _add_header(
                         file.suffix,
-                        parsed_args.license_id,
-                        parsed_args.company,
-                        parsed_args.year,
+                        license_id,
+                        company,
+                        year,
                     )
                     if header:
                         fp.seek(0)  # back to beginning of file
                         rest_of_file = fp.read()
                         fp.seek(0)
-                        fp.write(header)
-                        fp.write("\n")
-                        fp.write(rest_of_file)
+                        fp.write(header + "\n" + rest_of_file)
                         print(f"{file}: Added license header.")
-                        return 0
+                        return
+
                 except ValueError:
                     print(
                         f"{file}: No license header for the"
@@ -213,32 +194,24 @@ def _update_file(
                     )
                 except FileNotFoundError:
                     print(
-                        f"{file}: License file for {parsed_args.license_id} "
+                        f"{file}: License file for {license_id} "
                         "is not existing."
                     )
-                return 1
-            # old header existing - cleanup?
-            if cleanup_regexes:
-                old_content = file.read_text(encoding="utf-8")
-                new_content = _remove_outdated(
-                    content=old_content, cleanup_regexes=cleanup_regexes
-                )
-                if new_content:
-                    file.write_text(new_content, encoding="utf-8")
-                    print(f"{file}: Cleaned up!")
+                return
+
             # replace found header and write it to file
             if copyright_match and (
-                not copyright_match["modification_year"]
-                and copyright_match["creation_year"] < parsed_args.year
-                or copyright_match["modification_year"]
-                and copyright_match["modification_year"] < parsed_args.year
+                not copyright_match.modification_year
+                and copyright_match.creation_year < year
+                or copyright_match.modification_year
+                and copyright_match.modification_year < year
             ):
                 copyright_term = (
                     f"SPDX-FileCopyrightText: "
-                    f'{copyright_match["creation_year"]}'
-                    f'-{parsed_args.year} {copyright_match["company"]}'
+                    f"{copyright_match.creation_year}"
+                    f"-{year} {company}"
                 )
-                new_line = re.sub(regex, copyright_term, line)
+                new_line = re.sub(copyright_regex, copyright_term, line)
                 fp_write = fp.tell() - len(line)  # save position to insert
                 rest_of_file = fp.read()
                 fp.seek(fp_write)
@@ -250,24 +223,32 @@ def _update_file(
                 fp.truncate()
                 print(
                     f"{file}: Changed License Header Copyright Year "
-                    f'{copyright_match["modification_year"]} -> '
-                    f"{parsed_args.year}"
+                    f"{copyright_match.modification_year} -> "
+                    f"{year}"
                 )
 
             else:
                 print(f"{file}: License Header is ok.")
-            return 0
     except FileNotFoundError as e:
         print(f"{file}: File is not existing.")
         raise e
     except UnicodeDecodeError as e:
         print(f"{file}: Ignoring binary file.")
         raise e
+    # old header existing - cleanup?
+    if cleanup_regexes:
+        old_content = file.read_text(encoding="utf-8")
+        new_content = _remove_outdated_lines(
+            content=old_content, cleanup_regexes=cleanup_regexes
+        )
+        if new_content:
+            file.write_text(new_content, encoding="utf-8")
+            print(f"{file}: Cleaned up!")
 
 
 def _get_exclude_list(
-    exclude_file: Path, directories: List[Path]
-) -> List[Path]:
+    exclude_file: Path, directories: list[Path]
+) -> list[Path]:
     """Tries to get the list of excluded files / directories.
     If a file is given, it will be used. Otherwise it will be searched
     in the executed root path.
@@ -277,12 +258,12 @@ def _get_exclude_list(
 
     if exclude_file is None:
         exclude_file = Path(".pontos-header-ignore")
-    try:
-        exclude_lines = exclude_file.read_text(encoding="utf-8").split("\n")
-    except FileNotFoundError:
-        print("No exclude list file found.")
+
+    if not exclude_file.is_file():
         return []
 
+    exclude_lines = exclude_file.read_text(encoding="utf-8").splitlines()
+
     expanded_globs = [
         directory.rglob(line.strip())
         for directory in directories
@@ -302,114 +283,32 @@ def _get_exclude_list(
     return exclude_list
 
 
-def _parse_args(args=None):
-    """Parsing the args"""
-
-    parser = ArgumentParser(
-        description="Update copyright in source file headers.",
-        prog="pontos-update-header",
-    )
-
-    parser.add_argument(
-        "--quiet",
-        "-q",
-        action="store_true",
-        help="Don't print messages to the terminal",
-    )
-
-    parser.add_argument(
-        "--log-file",
-        dest="log_file",
-        type=str,
-        help="Acivate logging using the given file path",
-    )
-
-    date_group = parser.add_mutually_exclusive_group()
-    date_group.add_argument(
-        "-c",
-        "--changed",
-        action="store_true",
-        default=False,
-        help=(
-            "Update modified year using git log modified year. "
-            "This will not changed all files to current year!"
-        ),
-    )
-    date_group.add_argument(
-        "-y",
-        "--year",
-        default=str(datetime.now().year),
-        help=(
-            "If year is set, modified year will be "
-            "set to the specified year."
-        ),
-    )
-
-    parser.add_argument(
-        "-l",
-        "--license",
-        dest="license_id",
-        choices=SUPPORTED_LICENCES,
-        default="GPL-3.0-or-later",
-        help=("Use the passed license type"),
-    )
-
-    parser.add_argument(
-        "--company",
-        default="Greenbone AG",
-        help=(
-            "If a header will be added to file, "
-            "it will be licensed by company."
-        ),
-    )
-
-    files_group = parser.add_mutually_exclusive_group(required=True)
-    files_group.add_argument(
-        "-f", "--files", nargs="+", help="Files to update."
-    )
-    files_group.add_argument(
-        "-d",
-        "--directories",
-        nargs="+",
-        help="Directories to find files to update recursively.",
-    )
-
-    parser.add_argument(
-        "--exclude-file",
-        help=(
-            "File containing glob patterns for files to "
-            "ignore when finding files to update in a directory. "
-            "Will look for '.pontos-header-ignore' in the directory "
-            "if none is given. "
-            "The ignore file should only contain relative paths like *.py,"
-            "not absolute as **/*.py"
-        ),
-        type=FileType("r"),
-    )
-
-    parser.add_argument(
-        "--cleanup",
-        action="store_true",
-        default=False,
-        help="Do a cleanup: Remove lines from outdated header format",
-    )
-
-    return parser.parse_args(args)
-
-
-def _compile_outdated_regex() -> List[re.Pattern]:
+@cache
+def _compile_outdated_regex() -> list[re.Pattern]:
     """prepare regex patterns to remove old copyright lines"""
-    regexes: List[re.Pattern] = []
-    for line in OLD_LINES:
-        regexes.append(re.compile(rf"^(([#*]|//) ?)?{line}"))
-    return regexes
+    return [re.compile(rf"^(([#*]|//) ?)?{line}") for line in OLD_LINES]
 
 
-def main() -> None:
-    parsed_args = _parse_args()
+@cache
+def _compile_copyright_regex() -> re.Pattern:
+    """prepare the copyright regex"""
+    c_str = r"(SPDX-FileCopyrightText:|[Cc]opyright)"
+    d_str = r"(19[0-9]{2}|20[0-9]{2})"
+
+    return re.compile(rf"{c_str}.*? {d_str}?-? ?{d_str}? (.+)")
+
+
+def main(args: Optional[Sequence[str]] = None) -> None:
+    parsed_args = parse_args(args)
     exclude_list = []
+    year: str = parsed_args.year
+    license_id: str = parsed_args.license_id
+    company: str = parsed_args.company
+    changed: bool = parsed_args.changed
+    quiet: bool = parsed_args.quiet
+    cleanup: bool = parsed_args.cleanup
 
-    if parsed_args.quiet:
+    if quiet:
         term: Union[NullTerminal, RichTerminal] = NullTerminal()
     else:
         term = RichTerminal()
@@ -443,25 +342,26 @@ def main() -> None:
         term.error("Specify files to update!")
         sys.exit(1)
 
-    regex: re.Pattern = re.compile(
-        "(SPDX-FileCopyrightText:|[Cc]opyright).*?(19[0-9]{2}|20[0-9]{2}) "
-        f"?-? ?(19[0-9]{{2}}|20[0-9]{{2}})? ({parsed_args.company})"
-    )
-    cleanup_regexes: Optional[List[re.Pattern]] = None
-    if parsed_args.cleanup:
-        cleanup_regexes = _compile_outdated_regex()
-
     for file in files:
+        if changed:
+            try:
+                year = _get_modified_year(file)
+            except PontosError:
+                term.warning(
+                    f"{file}: Could not get date of last modification"
+                    f" via git, using {year} instead."
+                )
+
         try:
             if file.absolute() in exclude_list:
                 term.warning(f"{file}: Ignoring file from exclusion list.")
             else:
-                _update_file(
-                    file=file,
-                    regex=regex,
-                    parsed_args=parsed_args,
-                    term=term,
-                    cleanup_regexes=cleanup_regexes,
+                update_file(
+                    file,
+                    year,
+                    license_id,
+                    company,
+                    cleanup=cleanup,
                 )
         except (FileNotFoundError, UnicodeDecodeError, ValueError):
             continue
